<template>
  <div
    ref="target"
    class="h-full"
  >
    <slot v-bind="{ data, fetch }" />
  </div>
</template>

<script lang="ts" setup>
import useFetch from "@/js/contexts/shared/composables/useFetch";
import { merge, pickBy } from "lodash-es";
import type { Link } from "@/js/contexts/shared/models/Link";
import useVisibility from "@/js/contexts/shared/composables/useVisibility";
import type { UseFilters } from "@/js/contexts/shared/composables/useFilters";
import type { LinkAnalyticsFilters } from "@/js/contexts/links/models/LinkAnalyticsFilters";

const props = defineProps<{
  metric: "clicks" | "referrers" | "platforms" | "devices" | "countries" | "browsers";
  link?: Link;
  isAdmin?: boolean;
  filters: UseFilters<LinkAnalyticsFilters>;
}>();

defineSlots<{
  default(props: { data: any | undefined; fetch: typeof fetch }): void;
}>();

const target = ref<HTMLElement>();
const data = ref<any>();
const fetch = useFetch(props.isAdmin ? `/api/admin/links/analytics` : `/api/links/analytics`, { isFetching: true });

const visibility = useVisibility(target, () => load());

const load = () =>
  fetch
    .get<{ data: any }>(pickBy(merge({ link_id: props.link?.id, metric: props.metric, timezone: Intl.DateTimeFormat().resolvedOptions().timeZone }, props.filters.data())))
    .then((response) => (data.value = response.data.data));

watch(
  () => props.filters.data(),
  () => (visibility.viewed ? load() : void 0),
  { deep: true },
);
</script>
